package PureMessage::Redis::Service;
use strict;
use warnings;
use Carp;
use ActiveState::Handy qw(run);
use ActiveState::Unix::ProcInfo qw(proc_info);
use PureMessage::Version qw($PREFIX);
use PureMessage::Config qw(expand_filename);

use base 'Exporter';
our @EXPORT_OK = qw(
    redis_pid
    redis_is_running
    
    redis_start
    redis_stop
    redis_restart
    redis_status
);

my $CONFIG_FILE = "$PREFIX/etc/redis.conf";
my ($pidfile, $port ) = (undef, undef );

open (my $CF, '<', $CONFIG_FILE) || die "Can't open file $CONFIG_FILE: $!";
while ( my $l = <$CF> ) {
	chomp($l);
    $l =~ s/^\s+//;
    $l =~ s/\s+$//;
	
	next if ( $l =~ /"#"/ );
	$pidfile = $1 if ($l =~ /^pidfile (.*)$/i );
	$port    = $1 if ($l =~ /^port (\d{1,5})$/i );
	last if ($pidfile && $port);
}
$pidfile = "$PREFIX/var/run/redis.pid" unless $pidfile;
$port    = 6379 unless $port;

sub redis_pid{
    my $pid_file = expand_filename($pidfile);
    my $pid = 0;
    return $pid unless -e $pid_file;
    if(open(PF, '<', $pid_file)){
      $pid = <PF>;
      chomp($pid);
      close PF;
    }
    return $pid;
}

sub redis_is_running{
    my $pid = redis_pid();
    return 0 unless $pid;
    my $info = proc_info ('root_pid' => $pid);
    return 0 unless $info && $$info{$pid}; 
    return 1;
}

sub redis_start{
    return 1 if redis_is_running();

    my $cmd  = expand_filename('redis/bin/redis-server');
    my $conf = expand_filename('etc/redis.conf');
    unless( -x $cmd) {
        croak('Redis program does not have permissions to run.');
    }

    my $rc = 1;
    # eval {$rc = run("\@$cmd $conf >/dev/null 2>&1"); };
    eval {$rc = run("\@$cmd $conf >>$PREFIX/var/log/redis.log 2>&1"); };
	return 0 if ($rc || $@);

    # Poll for redis-server to start.
    my $count = 10;
    while ($count--) {
        sleep(1);
        if (redis_is_running()) {
          return 1;
        }
    }

    return 0;
}

sub redis_stop{
    return 1 unless redis_is_running();
    my $pid = redis_pid();
    my $count = 30;
    if(kill(0, $pid)){
        kill('TERM', $pid);
        while($count and kill(0,$pid)) {
          $count--;  
          sleep(1);
        }
        return $count;
    }
    return 0;
}

sub redis_restart{
  my $c = 1;
  if(redis_is_running()){
    $c = redis_stop();
  }
  return 0 unless $c;
  return redis_start();
}

sub redis_status{
  if (redis_is_running()) {
    print "Redis:\n";
    print "\tpid :     ".redis_pid()."\n";
    print "\tport:     $port\n";
    return 1;
  } else {
    print "Redis is not running.\n";
    return 0;
  }
}

1;

